@@ -19,9 +19,9 @@
 #' 
 #' One problem a user may encounter using the Catalog of Fishes website is that the input taxon name must match directly to a term in the database 
 #' or the database will not return any information. While this remains true using this package, users can attempt to resolve names by setting resolve = TRUE.
-#' When resolve = TRUE, rcatfish_search will use the Global Names Resolver (GNR) in an attempt to resolve the name, which will then be passed to downstream function calls.
-#' This is meant to be useful, but we recommend using this option be cautious about what the GNR returns. A message will print to the screen notifying you what 
-#' name the GNR resolved to be the best match and will be used, though we strongly recommend users check the resolved name does not deviate from their expectations (i.e. a homonym or similar name for a different group is not returned).
+#' When resolve = TRUE, rcatfish_search will use the Global Names Verifier (GNverifier) in an attempt to resolve the name, which will then be passed to downstream function calls.
+#' This is meant to be useful, but we recommend using this option be cautious about what the GNverifier returns. A message will print to the screen notifying you what 
+#' name the GNverifier resolved to be the best match and will be used, though we strongly recommend users check the resolved name does not deviate from their expectations (i.e. a homonym or similar name for a different group is not returned).
 #' @return Data frames. If taxon.history = TRUE, a list of two data frames. In this case, the first data frame TaxonSummary contains information on the description and current
 #' status of the taxa in the query, references to descriptions, and information on the type locality, types, family/subfamily, distribution, and habitat for species and type species
 #' gender, status, and authorities for genera searches. The second data frame, TaxonHistory contains detailed information on the taxonomic history 
@@ -58,6 +58,8 @@
 #'   \item Notes - Character. Any notes related to the taxon or taxonomic history (e.g. treated as a subspecies, availability of name, authorship issues, etc.).
 #'   \item AsSubgenus - Character. If the taxon was described as a subgenus of another genus, provides information on the genus.
 #'   \item Infrasubspecific - Character. Infrasubspecific designation if it exists. 
+#'   \item Miscellaneous - Character. Additional information not captured in other columns.
+#'   \item ResultCode - Character. A unique identifier for each nominal taxa, useful for linking table rows when homonyms are present.
 #' }
 #' @examples
 #' #Note that for Windows OS, OpenSSL must be used as a backend for curl. 
@@ -104,7 +106,7 @@
 #' @references 
 #' Fricke, R., Eschmeyer, W.N. & van der Laan, R. (Year Accessed). Eschmeyer's Catalog of Fishes: Genera, Species, References. https://researcharchive.calacademy.org/research/ichthyology/catalog/fishcatmain.asp
 #' 
-#' http://gnrd.globalnames.org/api http://gnrd.globalnames.org/
+#' Mozzherin, D. (2025). GNverifier -- a reconciler and resolver of scientific names against more than 100 data sources. (v1.3.0). Zenodo. https://doi.org/10.5281/zenodo.17245658
 #' @author Samuel R. Borstein, Brandon E. Dominy, Brian C. O'Meara 
 #' @export
 
@@ -159,12 +161,12 @@ rcatfish_search_species <- function(query, unavailable = FALSE, taxon.history =
   sleep.time <- ifelse(length(query)>1, sleep.time, 0) # Set sleep time for queries longer than 1
   
   #Make Tables to store results
-  tax_sum_columns <- c("Query",	"NominalTaxa",	"Author",	"DescriptionRef", "RefPage", "DescriptionYear", "Status", "CurrentNomenclature", "CurrentAuthority",	"Holotype",	"Paratype",	"Lectotype",	"Paralectotype",	"Neotype",	"Syntype", "NoTypes", "TypeLocality",	"Family","Subfamily",	"Distribution", "Fresh","Brackish", "Marine", "IUCNYear", "IUCNStatus", "NomenclatureNotes", "Infrasubspecific", "Miscellaneous")
+  tax_sum_columns <- c("Query",	"NominalTaxa",	"Author",	"DescriptionRef", "RefPage", "DescriptionYear", "Status", "CurrentNomenclature", "CurrentAuthority",	"Holotype",	"Paratype",	"Lectotype",	"Paralectotype",	"Neotype",	"Syntype", "NoTypes", "TypeLocality",	"Family","Subfamily",	"Distribution", "Fresh","Brackish", "Marine", "IUCNYear", "IUCNStatus", "NomenclatureNotes", "Infrasubspecific", "Miscellaneous", "ResultCode")
   all.tax.sum <- data.frame(matrix(nrow = 0, ncol = length(tax_sum_columns)),stringsAsFactors = FALSE)
   colnames(all.tax.sum) <- tax_sum_columns
   
   if (taxon.history == T) {
-    tax_hist_columns <- c("Query","NominalTaxa","Status","RecognizedNomenclature","Authority","RefNo","Notes")
+    tax_hist_columns <- c("Query","NominalTaxa","Status","RecognizedNomenclature","Authority","RefNo","Notes", "ResultCode")
     all.taxon.history <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns)))
     colnames(all.taxon.history) <- tax_hist_columns 
   }
@@ -175,6 +177,7 @@ rcatfish_search_species <- function(query, unavailable = FALSE, taxon.history =
     }
     
     local.result  <- catalog_search(query[TaxonIndex], type = "Species", unavailable, resolve, phrase)
+    ptime_Start <- Sys.time()
     if (length(local.result) > 0) {
       
       current.query.dat <- data.frame(matrix(nrow = length(local.result),ncol = length(tax_sum_columns)),stringsAsFactors = FALSE)#Set up data storage
@@ -196,6 +199,7 @@ rcatfish_search_species <- function(query, unavailable = FALSE, taxon.history =
         current.query.dat[result.index, c(3, 27)] <- get_infrasub(local.author = current.query.dat[result.index, 3]) #Get infrasubspecific tags
         current.query.dat[result.index, 5] <- get_page_number(local.result = local.result[result.index], processed.input = current.query.dat[result.index, 3])
         current.query.dat[result.index, 28] <- get_miscellanea(local.result[result.index], processed.input = current.query.dat[result.index,])
+        current.query.dat[result.index, 29] <- paste(TaxonIndex, result.index, sep = "_")
         wrongNAindex <- which(current.query.dat == "NA", arr.ind = T)
         current.query.dat[wrongNAindex] <- NA
       }
@@ -213,6 +217,7 @@ rcatfish_search_species <- function(query, unavailable = FALSE, taxon.history =
             ##### Process Nominal and Current Taxon History #####
             taxon.history.status$Query <- current.query.dat$Query[result.index] # Add query info
             taxon.history.status$NominalTaxa <-current.query.dat$NominalTaxa[result.index] # Add focal name
+            taxon.history.status$ResultCode <- current.query.dat$ResultCode[result.index]
             if (is.na(current.query.dat$Status[result.index])) {
               taxon.history.status$Status <- c("Nominal Species", "Currently no status")
             } else if (current.query.dat$Status[result.index] == "Uncertain") {
@@ -261,12 +266,13 @@ rcatfish_search_species <- function(query, unavailable = FALSE, taxon.history =
               references <- tax_hist_changes(changes[change.index])
               nom.refs <- references[[1]]
               nom.refs.no <- references[[2]]
-
+              
               current.taxon.history <- data.frame(matrix(nrow = ifelse(length(nom.refs) > 0, length(nom.refs), 0),ncol = length(tax_hist_columns)),stringsAsFactors = FALSE)#for the current history store data
               colnames(current.taxon.history) <- tax_hist_columns
               
               current.taxon.history$Query <-query[TaxonIndex]#fill with query info
               current.taxon.history$NominalTaxa <- current.query.dat$NominalTaxa[result.index]#fill in focal name
+              current.taxon.history$ResultCode <- current.query.dat$ResultCode[result.index]
               current.taxon.history$Status <- tax.status#add status of taxon
               current.taxon.history$Notes <- ifelse(notes == "",NA, notes)#add in notes on status
               current.taxon.history$RecognizedNomenclature <- ifelse(length(nom.status)==2, paste(nom.status[1], "x", nom.status[2]), nom.status)#add in recognized name
@@ -289,7 +295,10 @@ rcatfish_search_species <- function(query, unavailable = FALSE, taxon.history =
     } else if (length(local.result) == 0) {
       warning(paste0("No results found for supplied taxon ",query[TaxonIndex]))#Warn users if query is not found
     }
-    Sys.sleep(sleep.time)#system sleep between calls
+    ptime_End <- Sys.time()
+    ptime_Total <- ptime_End - ptime_Start
+    sleep.time <- ifelse(ptime_Total < 10, 10-ptime_Total, 0) #Adjust sleep time between server calls based processing time
+    if (TaxonIndex != length(query)) {Sys.sleep(sleep.time)} #System sleep between calls
   }
   
   if (taxon.history == TRUE) {
@@ -334,12 +343,12 @@ rcatfish_search_genus <- function(query, unavailable = FALSE, taxon.history = FA
   sleep.time <- ifelse(length(query)>1, sleep.time, 0) # Set sleep time for queries longer than 1
   
   #Make Tables to store results
-  tax_sum_columns <- c("Query",	"NominalTaxa",	"Author",	"DescriptionRef", "DescriptionYear", "Status", "CurrentNomenclature", "CurrentAuthority",	"TypeSpecies", "Gender", "TypeBy",	"Family","Subfamily","Notes", "NomenclatureNotes", "AsSubgenus")
+  tax_sum_columns <- c("Query",	"NominalTaxa",	"Author",	"DescriptionRef", "DescriptionYear", "Status", "CurrentNomenclature", "CurrentAuthority",	"TypeSpecies", "Gender", "TypeBy",	"Family","Subfamily","Notes", "NomenclatureNotes", "AsSubgenus", "ResultCode")
   all.tax.sum <- data.frame(matrix(nrow = 0, ncol = length(tax_sum_columns)),stringsAsFactors = FALSE)
   colnames(all.tax.sum) <- tax_sum_columns
   
   if (taxon.history == T) {
-    tax_hist_columns <- c("Query","NominalTaxa","Status","RecognizedNomenclature","Authority","RefNo","Notes")
+    tax_hist_columns <- c("Query","NominalTaxa","Status","RecognizedNomenclature","Authority","RefNo","Notes", "ResultCode")
     all.taxon.history <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns)))
     colnames(all.taxon.history) <- tax_hist_columns 
   }
@@ -350,94 +359,101 @@ rcatfish_search_genus <- function(query, unavailable = FALSE, taxon.history = FA
     }
     
     local.result  <- catalog_search(query[TaxonIndex], type = "Genus", unavailable, resolve, phrase)
+    ptime_Start <- Sys.time()
     
     if (length(local.result) > 0) {
-    current.query.dat <- data.frame(matrix(nrow = length(local.result),ncol = length(tax_sum_columns)),stringsAsFactors = FALSE)#Set up data storage
-    colnames(current.query.dat) <- tax_sum_columns
-    
-    current.query.dat$Query <- query[TaxonIndex]#Fill current query information
-    for(result.index in 1:length(local.result)){
-      #loop through results and parse data
-      current.query.dat[result.index, c(2:5,16)] <- get_focal_genus(local.result = local.result[result.index])#add in the focal genus, author and year
-      current.query.dat[result.index,6:8] <- get_current_status(local.result = local.result[result.index])#get genus status
-      current.query.dat$TypeSpecies[result.index] <- get_type_species(local.result = local.result[result.index])#get type species of genus
-      current.query.dat$Gender[result.index] <- get_gender(local.result = local.result[result.index])#get the gender of the genus
-      current.query.dat$TypeBy[result.index] <- get_type_by(local.result = local.result[result.index])#get what the type is by
-      families <- get_family_genus(local.result = local.result[result.index])#Get the family of the genus
-      ifelse(length(families)<2, current.query.dat[result.index,12:13] <- c(families, NA),current.query.dat[result.index,12:13] <- families)#check for subfamilies and write to table
-      current.query.dat$Notes[result.index] <- get_genus_notes(local.result = local.result[result.index])#Get notes on genus
-      current.query.dat$NomenclatureNotes[result.index] <- get_nomenclature_notes(local.result = local.result[result.index]) #Add in nomenclature notes (bold tags in search results)
-      wrongNAindex <- which(current.query.dat == "NA", arr.ind = T)
-      current.query.dat[wrongNAindex] <- NA
-    }
-    all.tax.sum <- rbind.data.frame(all.tax.sum,current.query.dat)#store current data in data to return
-    
-    if (taxon.history == TRUE) {
-      for (result.index in seq_along(local.result)) {
-        tryCatch(expr = {
-          ##### Set Up Data Storage Tables #####
-          taxon.history.dat <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns))) # Taxon history overall storage (will contain taxon.history.status and parsed.history.dat)
-          taxon.history.status <- data.frame(matrix(nrow = 2, ncol = length(tax_hist_columns))) # Nominal and current history storage
-          parsed.history.dat <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns))) # Parsed taxon history by catalog entry
-          colnames(taxon.history.dat) <- colnames(taxon.history.status) <- colnames(parsed.history.dat) <- tax_hist_columns
-          
-          ##### Process Nominal and Current Taxon History #####
-          taxon.history.status$Query <- current.query.dat$Query[result.index] # Add query info
-          taxon.history.status$NominalTaxa <-current.query.dat$NominalTaxa[result.index] # Add focal name
-          taxon.history.status$Status <- c("Nominal Species","Current") # Add status types
-          taxon.history.status$RecognizedNomenclature <- c(current.query.dat$NominalTaxa[result.index],current.query.dat$CurrentNomenclature[result.index]) # Add in recognized nomenclature
-          taxon.history.status$Authority <- c(current.query.dat$Author[result.index],current.query.dat$CurrentAuthority[result.index]) # Add authority info
-          taxon.history.status$RefNo <- c(current.query.dat$DescriptionRef[result.index], NA) # Add in description ref 
-          
-          ##### Process Taxon History #####
-          history <- qdapRegex::ex_between(local.result[result.index], c("\\u2022","\\u2022"), c(". <b>"," <b>")) # Extract bulleted information from catalog
-          if(is.na(history)) {
-            history <- qdapRegex::ex_between(local.result[result.index], c("\\u2022","\\u2022","\\u2022"), c("[A-z]+idae",". Distribution",". Habitat"), fixed = FALSE) # If history is not found due to lack of status, base off of family, distribution, or habitat
-          }
-          changes <- unlist(stringr::str_split(history, "\\u2022")) # Split history records on bullets
-          changes <- gsub("et al,","et al.",changes) # Handle issues of commas in et al.      
-          
-          for (change.index in seq_along(changes)) { # Process each bullet one by one
-            tax.status <- qdapRegex::ex_between(changes[change.index],left = c("",""),right = c("<i>"," --"))[[1]][1] # Obtain taxa status for current bullet
-            nom.status <- qdapRegex::ex_between(changes[change.index],left = "<i>",right = "</i>")[[1]][1] # Obtain the nomenclature status
-            # Find any notes before authorship information for current bullet
-            notes <- gsub(pattern = "^[12]\\d\\d\\d\\)?\\,?| --|<i>|</i>", replacement = "",
-                          x = stringr::str_extract(string = changes[change.index], pattern = "[12]\\d\\d\\d\\)?.* --"))
-            
-            references <- tax_hist_changes(changes[change.index])
-            nom.refs <- references[[1]]
-            nom.refs.no <- references[[2]]
-            
-            current.taxon.history <- data.frame(matrix(nrow = ifelse(length(nom.refs) > 0, length(nom.refs), 0),ncol = length(tax_hist_columns)),stringsAsFactors = FALSE)#for the current history store data
-            colnames(current.taxon.history) <- tax_hist_columns
-            
-            current.taxon.history$Query <-query[TaxonIndex]#fill with query info
-            current.taxon.history$NominalTaxa <- current.query.dat$NominalTaxa[result.index]#fill in focal name
-            current.taxon.history$Status <- tax.status#add status of taxon
-            current.taxon.history$Notes <- ifelse(notes == "",NA, notes)#add in notes on status
-            current.taxon.history$RecognizedNomenclature <- nom.status#add in recognized name
-            current.taxon.history$Authority <- ifelse(nchar(nom.refs) > 0, nom.refs, NA)#add in authority info
-            current.taxon.history$RefNo <- nom.refs.no#put in reference number
-            parsed.history.dat<-rbind(parsed.history.dat,current.taxon.history)#combine with other taxon history data
-          }
-          taxon.history.dat<-rbind(taxon.history.status[1,],parsed.history.dat,taxon.history.status[2,])#combine current taxon history with current/nominal data
-          if(taxon.history == T) {all.taxon.history <- rbind.data.frame(all.taxon.history, taxon.history.dat)}#combine current history with all history to be returned
-          RowsToDelete <- which(is.na(all.taxon.history$TaxonHistory$Status) & is.na(all.taxon.history$TaxonHistory$RecognizedNomenclature) & is.na(all.taxon.history$TaxonHistory$Authority) & is.na(all.taxon.history$TaxonHistory$RefNo) & is.na(all.taxon.history$TaxonHistory$Notes))
-          if (length(RowsToDelete) > 0) {all.taxon.history <- all.taxon.history[-RowsToDelete,]}
-        },
-        error = function(cond) {
-          message(paste("Error when parsing taxon history for", current.query.dat$NominalTaxa[result.index]))
-        })
+      current.query.dat <- data.frame(matrix(nrow = length(local.result),ncol = length(tax_sum_columns)),stringsAsFactors = FALSE)#Set up data storage
+      colnames(current.query.dat) <- tax_sum_columns
+      
+      current.query.dat$Query <- query[TaxonIndex]#Fill current query information
+      for(result.index in 1:length(local.result)){
+        #loop through results and parse data
+        current.query.dat[result.index, c(2:5,16)] <- get_focal_genus(local.result = local.result[result.index])#add in the focal genus, author and year
+        current.query.dat[result.index,6:8] <- get_current_status(local.result = local.result[result.index])#get genus status
+        current.query.dat$TypeSpecies[result.index] <- get_type_species(local.result = local.result[result.index])#get type species of genus
+        current.query.dat$Gender[result.index] <- get_gender(local.result = local.result[result.index])#get the gender of the genus
+        current.query.dat$TypeBy[result.index] <- get_type_by(local.result = local.result[result.index])#get what the type is by
+        families <- get_family_genus(local.result = local.result[result.index])#Get the family of the genus
+        ifelse(length(families)<2, current.query.dat[result.index,12:13] <- c(families, NA),current.query.dat[result.index,12:13] <- families)#check for subfamilies and write to table
+        current.query.dat$Notes[result.index] <- get_genus_notes(local.result = local.result[result.index])#Get notes on genus
+        current.query.dat$NomenclatureNotes[result.index] <- get_nomenclature_notes(local.result = local.result[result.index]) #Add in nomenclature notes (bold tags in search results)
+        current.query.dat$ResultCode[result.index] <- paste(TaxonIndex, result.index, sep = "_")
+        wrongNAindex <- which(current.query.dat == "NA", arr.ind = T)
+        current.query.dat[wrongNAindex] <- NA
       }
-      del <- which(is.na(all.taxon.history$Status))
-      RowsToDelete <- c()
-      for (i in seq_along(del)) {if (is.na(all.taxon.history[del[i], 4]) && is.na(all.taxon.history[del[i], 5]) && is.na(all.taxon.history[del[i], 6]) && is.na(all.taxon.history[del[i], 7])) {RowsToDelete <- c(RowsToDelete, del[i])}}
-      if(length(RowsToDelete) > 0) {all.taxon.history <- all.taxon.history[-RowsToDelete,]}
+      all.tax.sum <- rbind.data.frame(all.tax.sum,current.query.dat)#store current data in data to return
+      
+      if (taxon.history == TRUE) {
+        for (result.index in seq_along(local.result)) {
+          tryCatch(expr = {
+            ##### Set Up Data Storage Tables #####
+            taxon.history.dat <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns))) # Taxon history overall storage (will contain taxon.history.status and parsed.history.dat)
+            taxon.history.status <- data.frame(matrix(nrow = 2, ncol = length(tax_hist_columns))) # Nominal and current history storage
+            parsed.history.dat <- data.frame(matrix(nrow = 0, ncol = length(tax_hist_columns))) # Parsed taxon history by catalog entry
+            colnames(taxon.history.dat) <- colnames(taxon.history.status) <- colnames(parsed.history.dat) <- tax_hist_columns
+            
+            ##### Process Nominal and Current Taxon History #####
+            taxon.history.status$Query <- current.query.dat$Query[result.index] # Add query info
+            taxon.history.status$NominalTaxa <-current.query.dat$NominalTaxa[result.index] # Add focal name
+            taxon.history.status$ResultCode <- current.query.dat$ResultCode[result.index]
+            taxon.history.status$Status <- c("Nominal Species","Current") # Add status types
+            taxon.history.status$RecognizedNomenclature <- c(current.query.dat$NominalTaxa[result.index],current.query.dat$CurrentNomenclature[result.index]) # Add in recognized nomenclature
+            taxon.history.status$Authority <- c(current.query.dat$Author[result.index],current.query.dat$CurrentAuthority[result.index]) # Add authority info
+            taxon.history.status$RefNo <- c(current.query.dat$DescriptionRef[result.index], NA) # Add in description ref 
+            
+            ##### Process Taxon History #####
+            history <- qdapRegex::ex_between(local.result[result.index], c("\\u2022","\\u2022"), c(". <b>"," <b>")) # Extract bulleted information from catalog
+            if(is.na(history)) {
+              history <- qdapRegex::ex_between(local.result[result.index], c("\\u2022","\\u2022","\\u2022"), c("[A-z]+idae",". Distribution",". Habitat"), fixed = FALSE) # If history is not found due to lack of status, base off of family, distribution, or habitat
+            }
+            changes <- unlist(stringr::str_split(history, "\\u2022")) # Split history records on bullets
+            changes <- gsub("et al,","et al.",changes) # Handle issues of commas in et al.      
+            
+            for (change.index in seq_along(changes)) { # Process each bullet one by one
+              tax.status <- qdapRegex::ex_between(changes[change.index],left = c("",""),right = c("<i>"," --"))[[1]][1] # Obtain taxa status for current bullet
+              nom.status <- qdapRegex::ex_between(changes[change.index],left = "<i>",right = "</i>")[[1]][1] # Obtain the nomenclature status
+              # Find any notes before authorship information for current bullet
+              notes <- gsub(pattern = "^[12]\\d\\d\\d\\)?\\,?| --|<i>|</i>", replacement = "",
+                            x = stringr::str_extract(string = changes[change.index], pattern = "[12]\\d\\d\\d\\)?.* --"))
+              
+              references <- tax_hist_changes(changes[change.index])
+              nom.refs <- references[[1]]
+              nom.refs.no <- references[[2]]
+              
+              current.taxon.history <- data.frame(matrix(nrow = ifelse(length(nom.refs) > 0, length(nom.refs), 0),ncol = length(tax_hist_columns)),stringsAsFactors = FALSE)#for the current history store data
+              colnames(current.taxon.history) <- tax_hist_columns
+              
+              current.taxon.history$Query <-query[TaxonIndex]#fill with query info
+              current.taxon.history$NominalTaxa <- current.query.dat$NominalTaxa[result.index]#fill in focal name
+              current.taxon.history$ResultCode <- current.query.dat$ResultCode[result.index]
+              current.taxon.history$Status <- tax.status#add status of taxon
+              current.taxon.history$Notes <- ifelse(notes == "",NA, notes)#add in notes on status
+              current.taxon.history$RecognizedNomenclature <- nom.status#add in recognized name
+              current.taxon.history$Authority <- ifelse(nchar(nom.refs) > 0, nom.refs, NA)#add in authority info
+              current.taxon.history$RefNo <- nom.refs.no#put in reference number
+              parsed.history.dat<-rbind(parsed.history.dat,current.taxon.history)#combine with other taxon history data
+            }
+            taxon.history.dat<-rbind(taxon.history.status[1,],parsed.history.dat,taxon.history.status[2,])#combine current taxon history with current/nominal data
+            if(taxon.history == T) {all.taxon.history <- rbind.data.frame(all.taxon.history, taxon.history.dat)}#combine current history with all history to be returned
+            RowsToDelete <- which(is.na(all.taxon.history$TaxonHistory$Status) & is.na(all.taxon.history$TaxonHistory$RecognizedNomenclature) & is.na(all.taxon.history$TaxonHistory$Authority) & is.na(all.taxon.history$TaxonHistory$RefNo) & is.na(all.taxon.history$TaxonHistory$Notes))
+            if (length(RowsToDelete) > 0) {all.taxon.history <- all.taxon.history[-RowsToDelete,]}
+          },
+          error = function(cond) {
+            message(paste("Error when parsing taxon history for", current.query.dat$NominalTaxa[result.index]))
+          })
+        }
+        del <- which(is.na(all.taxon.history$Status))
+        RowsToDelete <- c()
+        for (i in seq_along(del)) {if (is.na(all.taxon.history[del[i], 4]) && is.na(all.taxon.history[del[i], 5]) && is.na(all.taxon.history[del[i], 6]) && is.na(all.taxon.history[del[i], 7])) {RowsToDelete <- c(RowsToDelete, del[i])}}
+        if(length(RowsToDelete) > 0) {all.taxon.history <- all.taxon.history[-RowsToDelete,]}
+      }
+    } else if (length(local.result) == 0) {
+      warning(paste0("No results found for supplied taxon ",query[TaxonIndex]))#Warn users if query is not found
     }
-  } else if (length(local.result) == 0) {
-    warning(paste0("No results found for supplied taxon ",query[TaxonIndex]))#Warn users if query is not found
-  }
-    Sys.sleep(sleep.time)#system sleep between calls
+    ptime_End <- Sys.time()
+    ptime_Total <- ptime_End - ptime_Start
+    sleep.time <- ifelse(ptime_Total < 10, 10-ptime_Total, 0)
+    if (TaxonIndex != length(query)) {Sys.sleep(sleep.time)} #System sleep between calls
   }
   if (taxon.history == TRUE) {
     all.taxon.history.cleaned <- lapply(all.taxon.history, function(x) gsub("amp;", "", x))#kill &
@@ -448,4 +464,4 @@ rcatfish_search_genus <- function(query, unavailable = FALSE, taxon.history = FA
   } else {
     return(all.tax.sum)
   }
-}
+}
\ No newline at end of file
